@@ -8,8 +8,8 @@ use tokio_io::AsyncWrite;
 
 use zstd_safe;
 
-use dict::{DecoderDictionary, EncoderDictionary};
-use stream::{raw, zio};
+use crate::dict::{DecoderDictionary, EncoderDictionary};
+use crate::stream::{raw, zio};
 
 #[cfg(test)]
 #[cfg(feature = "tokio")]
@@ -124,7 +124,7 @@ impl<W: Write> Encoder<W> {
     /// but requires the dictionary to be present during decompression.)
     pub fn with_prepared_dictionary(
         writer: W,
-        dictionary: &EncoderDictionary,
+        dictionary: &EncoderDictionary<'_>,
     ) -> io::Result<Self> {
         let encoder = raw::Encoder::with_prepared_dictionary(dictionary)?;
         let writer = zio::Writer::new(writer, encoder);
@@ -211,6 +211,27 @@ impl<W: Write> Encoder<W> {
     pub fn recommended_input_size() -> usize {
         zstd_safe::cstream_in_size()
     }
+
+    /// Controls whether zstd should include a content checksum at the end of each frame.
+    pub fn include_checksum(
+        &mut self,
+        include_checksum: bool,
+    ) -> io::Result<()> {
+        self.writer.operation_mut().set_parameter(
+            zstd_safe::CParameter::ChecksumFlag(include_checksum),
+        )
+    }
+
+    /// Enables multithreaded compression
+    ///
+    /// * If `n_workers == 0` (default), then multithreaded will be disabled.
+    /// * If `n_workers >= 1`, then compression will be done in separate threads.
+    ///   So even `n_workers = 1` may increase performance by separating IO and compression.
+    pub fn multithread(&mut self, n_workers: u32) -> io::Result<()> {
+        self.writer
+            .operation_mut()
+            .set_parameter(zstd_safe::CParameter::NbWorkers(n_workers))
+    }
 }
 
 impl<W: Write> Write for Encoder<W> {
@@ -226,6 +247,8 @@ impl<W: Write> Write for Encoder<W> {
 #[cfg(feature = "tokio")]
 impl<W: AsyncWrite> AsyncWrite for Encoder<W> {
     fn shutdown(&mut self) -> Poll<(), io::Error> {
+        use tokio_io::try_nb;
+
         try_nb!(self.do_finish());
         self.writer.writer_mut().shutdown()
     }
@@ -253,7 +276,7 @@ impl<W: Write> Decoder<W> {
     /// but requires the dictionary to be present during decompression.)
     pub fn with_prepared_dictionary(
         writer: W,
-        dictionary: &DecoderDictionary,
+        dictionary: &DecoderDictionary<'_>,
     ) -> io::Result<Self> {
         let decoder = raw::Decoder::with_prepared_dictionary(dictionary)?;
         let writer = zio::Writer::new(writer, decoder);
@@ -280,7 +303,7 @@ impl<W: Write> Decoder<W> {
 
     /// Return a recommendation for the size of data to write at once.
     pub fn recommended_input_size() -> usize {
-        zstd_safe::cstream_in_size()
+        zstd_safe::dstream_in_size()
     }
 }
 
